/*
  common.cpp - Functions imported from Common.dll.

  Jason Hood, 1 August, 2009.

  22 December, 2018:
  * use the program's directory if the registry entry doesn't exist.
*/

#include "common.h"

#pragma comment( lib, "advapi32.lib" )
#pragma comment( lib, "user32.lib" )


// The registry entry containing the path to Freelancer.
#define FL_REG "SOFTWARE\\Microsoft\\Microsoft Games\\Freelancer\\1.0"


// Manually import the functions, to provide path independence.
static struct
{
  FARPROC addr;
  LPCSTR  name;
}
imports[] =
{
  NULL, "?CreateID@@YAIPBD@Z",
  NULL, "?MakeId@@YAGPBD@Z",
  NULL, "?MakeLocationID@@YAIIPBD@Z",
  NULL, "?GetUserDataPath@@YA_NQAD@Z",
  NULL, "?HexToWide@@YAXPBDPAG@Z",
  NULL, "?filetime_to_wstring@@YAPBGABU_FILETIME@@@Z",
  NULL, "??0INI_Reader@@QAE@XZ",
  NULL, "??1INI_Reader@@QAE@XZ",
  NULL, "?close@INI_Reader@@QAEXXZ",
  NULL, "?find_header@INI_Reader@@QAE_NPBD@Z",
  NULL, "?get_header_ptr@INI_Reader@@QAEPBDXZ",
  NULL, "?get_line_ptr@INI_Reader@@QAEPBDXZ",
  NULL, "?get_name_ptr@INI_Reader@@QAEPBDXZ",
  NULL, "?get_num_parameters@INI_Reader@@QBEIXZ",
  NULL, "?get_value_bool@INI_Reader@@QAE_NI@Z",
  NULL, "?get_value_float@INI_Reader@@QAEMI@Z",
  NULL, "?get_value_int@INI_Reader@@QAEHI@Z",
  NULL, "?get_value_string@INI_Reader@@QAEPBDI@Z",
  NULL, "?get_value_string@INI_Reader@@QAEPBDXZ",
  NULL, "?is_header@INI_Reader@@QAE_NPBD@Z",
  NULL, "?is_value@INI_Reader@@QAE_NPBD@Z",
  NULL, "?is_value_empty@INI_Reader@@QAE_NI@Z",
  NULL, "?open@INI_Reader@@QAE_NPBD_N@Z",
  NULL, "?open_memory@INI_Reader@@QAE_NPBDI@Z",
  NULL, "?read_header@INI_Reader@@QAE_NXZ",
  NULL, "?read_value@INI_Reader@@QAE_NXZ",
};


enum
{
  impCreateID,
  impMakeID,
  impMakeLocationID,
  impGetUserDataPath,
  impHexToWide,
  impfiletime_to_wstring,
  INI_Reader_ctor,
  INI_Reader_dtor,
  INI_Reader_close,
  INI_Reader_find_header,
  INI_Reader_get_header_ptr,
  INI_Reader_get_line_ptr,
  INI_Reader_get_name_ptr,
  INI_Reader_get_num_parameters,
  INI_Reader_get_value_bool,
  INI_Reader_get_value_float,
  INI_Reader_get_value_int,
  INI_Reader_get_value_stringI,
  INI_Reader_get_value_string,
  INI_Reader_is_header,
  INI_Reader_is_value,
  INI_Reader_is_value_empty,
  INI_Reader_open,
  INI_Reader_open_memory,
  INI_Reader_read_header,
  INI_Reader_read_value,
  IMPORTS
};


#define IMP( func, idx ) \
  __declspec(naked) func { __asm jmp imports[idx*8].addr }

IMP( UINT CreateID( LPCSTR ),			 impCreateID )
IMP( USHORT MakeID( LPCSTR ),			 impMakeID )
IMP( UINT MakeLocationID( UINT, LPCSTR ),	 impMakeLocationID )
IMP( bool GetUserDataPath( LPSTR const ),	 impGetUserDataPath )
IMP( void HexToWide( LPCSTR, LPWSTR ),		 impHexToWide )
IMP( LPCWSTR filetime_to_wstring( const FILETIME& ), impfiletime_to_wstring )
IMP( INI_Reader::INI_Reader(),			 INI_Reader_ctor )
IMP( INI_Reader::~INI_Reader(), 		 INI_Reader_dtor )
IMP( void INI_Reader::close(),			 INI_Reader_close )
IMP( bool INI_Reader::find_header( LPCSTR ),	 INI_Reader_find_header )
IMP( LPCSTR INI_Reader::get_header_ptr(),	 INI_Reader_get_header_ptr )
IMP( LPCSTR INI_Reader::get_line_ptr(), 	 INI_Reader_get_line_ptr )
IMP( LPCSTR INI_Reader::get_name_ptr(), 	 INI_Reader_get_name_ptr )
IMP( UINT INI_Reader::get_num_parameters() const,INI_Reader_get_num_parameters )
IMP( bool INI_Reader::get_value_bool( UINT ),	 INI_Reader_get_value_bool )
IMP( float INI_Reader::get_value_float( UINT ),  INI_Reader_get_value_float )
IMP( int INI_Reader::get_value_int( UINT ),	 INI_Reader_get_value_int )
IMP( LPCSTR INI_Reader::get_value_string( UINT ),INI_Reader_get_value_stringI )
IMP( LPCSTR INI_Reader::get_value_string(),	 INI_Reader_get_value_string )
IMP( bool INI_Reader::is_header( LPCSTR ),	 INI_Reader_is_header )
IMP( bool INI_Reader::is_value( LPCSTR ),	 INI_Reader_is_value )
IMP( bool INI_Reader::is_value_empty( UINT ),	 INI_Reader_is_value_empty )
IMP( bool INI_Reader::open( LPCSTR, bool ),	 INI_Reader_open )
IMP( bool INI_Reader::open_memory(LPCSTR, UINT), INI_Reader_open_memory )
IMP( bool INI_Reader::read_header(),		 INI_Reader_read_header )
IMP( bool INI_Reader::read_value(),		 INI_Reader_read_value )


// Change to Freelancer's EXE directory, read the imports from Common.dll (and
// load the resource DLLs found in freelancer.ini) and move back to the
// original directory.
Common::Common( bool loadres )
{
  // Retrieve the path from the registry.
  HKEY flpath;

  *path = '\0';
  if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, FL_REG, 0,
		    KEY_READ, &flpath ) == ERROR_SUCCESS)
  {
    DWORD size = sizeof(path);
    RegQueryValueEx( flpath, "AppPath", NULL, NULL, (LPBYTE)path, &size );
    RegCloseKey( flpath );
  }
  if (*path == '\0')
  {
    int len = GetModuleFileName( NULL, path, sizeof(path) );
    while (--len > 0)
    {
      if (path[len] == '\\')
      {
	path[len] = '\0';
	break;
      }
    }
  }

  // Need to do this in the EXE directory to find other DLLs.
  SetExeDirectory( true );
  common = LoadLibrary( "common.dll" );
  if (!common)
  {
    fprintf( stderr, "Failed to load %s\\EXE\\common.dll.\n", path );
    exit( 1 );
  }

  for (int i = 0; i < IMPORTS; ++i)
    imports[i].addr = GetProcAddress( common, imports[i].name );

  if (loadres)
  {
    resmod.push_back( LoadLibrary( "resources.dll" ) );
    resdll.push_back( "Resources" );

    INI_Reader ini;
    ini.open( "freelancer.ini" );
    ini.find_header( "Resources" );
    while (ini.read_value())
    {
      // Let's assume they're all "DLL".
      resmod.push_back( LoadLibrary( ini.get_value_string() ) );
      // Strip the extension.
      LPCSTR dll = ini.get_value_string();
      LPCSTR ext = strchr( dll, '.' );
      if (!ext)
	ext = strchr( dll, '\0' );
      resdll.push_back( string( dll, ext - dll ) );
    }
    ini.close();
  }

  SetExeDirectory( false );
}


Common::~Common()
{
  FreeLibrary( common );

  for (int i = resmod.size(); --i >= 0;)
    FreeLibrary( resmod[i] );
}


// Move to the Freelancer\EXE directory, or back to the prior directory (which
// assumes the move was previously made).
void Common::SetExeDirectory( bool exe )
{
  if (exe)
  {
    GetCurrentDirectory( sizeof(cwd), cwd );
    SetCurrentDirectory( path );
    SetCurrentDirectory( "EXE" );
  }
  else
    SetCurrentDirectory( cwd );
}
