/*
  common.h - Declarations for functions imported from Common.dll.

  Jason Hood, 1 August, 2009.
*/

#ifndef _COMMON_H
#define _COMMON_H

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <string>
#include <vector>

using std::string;

typedef std::vector<HMODULE> ResModVec;
typedef std::vector<string>  ResDllVec;


enum { CWD, FLROOT, FLEXE, FLDATA };

typedef void (*CBProcess_Ini)( LPCSTR );


class Common
{
public:
  Common( bool = false );
  ~Common();

  void	  SetExeDirectory( bool );
  LPCSTR  GetDirectory( int );

  void	  process_args( char* [], CBProcess_Ini, LPCSTR );

  int	  GetString( UINT, LPSTR, int, bool = true );
  LPCWSTR GetString( UINT, int& );
  LPCWSTR GetHTML( UINT, int& );
  LPCSTR  get_resdll( UINT, UINT& );

  bool	  one_file;

private:
  void	  search_dir( LPCSTR, bool );

  HMODULE   common;
  ResModVec resmod;
  ResDllVec resdll;
  char	    path[MAX_PATH];
  char	    cwd[MAX_PATH];
  char	    buf[MAX_PATH];
  CBProcess_Ini process_ini;
  LPCSTR pattern;
};


LPSTR  EngNum( double, LPSTR, bool );
LPCSTR get_file_name( LPCSTR );
UINT   get_hash( LPCSTR );

enum { fmtDefault, fmtCSV, fmtIONCROSS, fmtMTR };
enum { outName = 1, outHex = 2, outMinus = 4 };
void   output( UINT, LPCSTR, LPCSTR, int, int );


UINT	CreateID( LPCSTR );
USHORT	MakeID( LPCSTR );
UINT	MakeLocationID( UINT, LPCSTR );
bool	GetUserDataPath( LPSTR const );
void	HexToWide( LPCSTR, LPWSTR );
LPCWSTR filetime_to_wstring( const FILETIME& );


// The type returned by get_type.
enum { BINI_BOOL, BINI_INT, BINI_FLOAT, BINI_STRING };


// Use an abridged version of Freelancer's own ini-reading code.
class INI_Reader
{
public:
  INI_Reader();
  ~INI_Reader();
  void	 close();
  bool	 find_header( LPCSTR );
  LPCSTR get_header_ptr();
  LPCSTR get_line_ptr();
  LPCSTR get_name_ptr();
  UINT	 get_num_parameters() const;
  bool	 get_value_bool( UINT );
  float  get_value_float( UINT );
  int	 get_value_int( UINT );
  LPCSTR get_value_string( UINT );
  LPCSTR get_value_string();
  bool	 is_header( LPCSTR );
  bool	 is_value( LPCSTR );
  bool	 is_value_empty( UINT );
  bool	 open( LPCSTR, bool = false );
  bool	 open_memory( LPCSTR, UINT );
  bool	 read_header();
  bool	 read_value();

  bool	 is_bini() const { return *(bool*)(data+0x1564); }

  // Assumes (!is_bini())
  UINT	 fsize()   const { return *(UINT*)(data+0x1c); }

  // Assumes (is_bini() && idx < get_num_parameters())
  BYTE	 get_type( UINT idx ) const { return (*(BYTE**)(data+0x1460))[idx*5]; }

private:
  BYTE	 data[0x1568];
};

#endif
