/*
  common_args.cpp - Implementation of Common's argument processing.

  Jason Hood, 5 August, 2009.
*/

#include "common.h"


// Call PROC for each file in the NULL-terminated ARGV list, or if it is
// a directory and PAT is not NULL, for all PAT files in it and its subdirs.
// If no args are given and PAT is not NULL, search the Freelancer DATA dir.
// The public variable ONE_FILE is set true if only one file arg has been given.
void Common::process_args( char* argv[], CBProcess_Ini proc, LPCSTR pat )
{
  process_ini = proc;
  pattern     = pat;

  if (!*argv)
  {
    if (pattern)
    {
      one_file = false;
      search_dir( GetDirectory( FLDATA ), true );
    }
  }
  else
  {
    one_file = (!argv[1]);
    do
    {
      if (GetFileAttributes( *argv ) & FILE_ATTRIBUTE_DIRECTORY)
      {
	if (pattern)
	{
	  one_file = false;
	  search_dir( *argv, true );
	}
      }
      else
      {
	process_ini( *argv );
      }
    } while (*++argv);
  }
}


// Process the files in the given directory and all its subdirs. If progress is
// true, display the name of the subdirectory being searched.
void Common::search_dir( LPCSTR dir, bool progress )
{
  WIN32_FIND_DATA fd;
  HANDLE find;
  char	 name[MAX_PATH];
  int	 len, spcs = 12;

  len = strlen( dir ) + 1;
  _snprintf( name, sizeof(name), "%s\\%s", dir, pattern );
  find = FindFirstFile( name, &fd );
  if (find != INVALID_HANDLE_VALUE)
  do
  {
    strcpy( name+len, fd.cFileName );
    process_ini( name );
  } while (FindNextFile( find, &fd ));
  FindClose( find );

  strcpy( name+len, "*" );
  find = FindFirstFile( name, &fd );
  if (find != INVALID_HANDLE_VALUE)
  do
  {
    if ((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
	&& !(fd.cFileName[0] == '.' && (fd.cFileName[1] == '\0' ||
	     (fd.cFileName[1] == '.' && fd.cFileName[2] == '\0'))))
    {
      if (progress)
	spcs = fprintf( stderr, "Processing %-*s\r", spcs - 11, fd.cFileName );
      strcpy( name+len, fd.cFileName );
      search_dir( name, false );
    }
  } while (FindNextFile( find, &fd ));
  FindClose( find );

  if (progress)
    fprintf( stderr, "%*c\r", spcs, ' ' );
}
