/*
  engnum.cpp - Implementation of EngNum.

  Jason Hood, 5 August, 2009.
*/

#include "common.h"
#include <cmath>


// Convert a number like "%g", but use engineering notation (the exponent is a
// multiple of 3; there's also no plus sign or leading zeros).  If trailing is
// true, append ".0" to numbers without a point (and exponent).
LPSTR EngNum( double num, LPSTR buf, bool trailing )
{
  LPSTR s = buf;

  if (num < 0)
  {
    *s++ = '-';
    num = -num;
  }

  if (num && (num >= 1e6 || num <= 1e-5))
  {
    int exp = (int)floor( log10( num ) );
    int adj = exp % 3;
    if (adj < 0)
      adj += 3;
    exp -= adj;
    num /= pow( 10, exp );
    if (num > 999.999)		// some numbers just aren't what they seem...
    {
      num /= 1000;
      exp += 3;
    }
    sprintf( s, "%ge%d", num, exp );
  }
  else
  {
    int len = sprintf( s, "%g", num );
    if (trailing && !strchr( s, '.' ))
      strcpy( s+len, ".0" );
  }

  return buf;
}
