/*
  hashdb.cpp - Implementation of HashDB.

  Jason Hood, 5 August, 2009.
*/

#include "common.h"
#include "hashdb.h"


// Open the hash database file NAME.  If it can't be opened, and NAME has no
// path, see if it exists in the same directory as the program.  If TEST is
// true, display nicknames that both hash to the same value.
bool HashDB::open( LPCSTR name, bool test )
{
  char temp[MAX_PATH];

  file = CreateFile( name, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING,
		     0, NULL );
  if (file == INVALID_HANDLE_VALUE)
  {
    if (name[1] != ':' && !strpbrk( name, "\\/" ))
    {
      LPCSTR n = get_file_name( _pgmptr );
      if (n != _pgmptr)
      {
	_snprintf( temp, sizeof(temp), "%.*s%s", n - _pgmptr, _pgmptr, name );
	name = temp;
	file = CreateFile( name, GENERIC_READ, FILE_SHARE_READ, NULL,
			   OPEN_EXISTING, 0, NULL );
      }
    }
  }
  if (file == INVALID_HANDLE_VALUE)
  {
    fprintf( stderr, "Unable to open \"%s\".\n", name );
    return false;
  }
  map = CreateFileMapping( file, NULL, PAGE_READONLY, 0, 0, NULL );
  if (map == NULL)
  {
    fprintf( stderr, "Unable to create file mapping for \"%s\".\n", name );
    CloseHandle( file );
    return false;
  }
  view = MapViewOfFile( map, FILE_MAP_READ, 0, 0, 0 );
  if (view == NULL)
  {
    fprintf( stderr, "Unable to map view of \"%s\".\n", name );
    CloseHandle( map );
    CloseHandle( file );
    return false;
  }

  union
  {
    LPUINT id;
    LPCSTR str;
  } ptr;
  ptr.id = (LPUINT)view;
  if (*ptr.id != '1HLF')
  {
    fprintf( stderr, "Unsupported file format for \"%s\".\n", name );
    close();
    return false;
  }
  LPUINT end = (LPUINT)((LPBYTE)ptr.id++ + GetFileSize( file, NULL ) - 4);
  while (ptr.id < end)
  {
    UINT id = *ptr.id++;
    NickName nickname;
    nickname.first = ptr.str;
    ptr.str += strlen( ptr.str ) + 1;
    nickname.second = ptr.str;
    ptr.str += strlen( ptr.str ) + 1;
    if (test && find( id ))
      fprintf( stderr, "%s and %s both hash to %u.\n",
		       fnd->second.first, nickname.first, id );
    hashes[id] = nickname;
  }

  return true;
}


void HashDB::close()
{
  UnmapViewOfFile( view );
  CloseHandle( map );
  CloseHandle( file );
}
